from typing import Any, Dict, List, Optional

from botocore import xform_name as xform_name
from botocore.exceptions import BotoCoreError as BotoCoreError
from botocore.exceptions import ConnectionError as ConnectionError
from botocore.exceptions import HTTPClientError as HTTPClientError
from botocore.model import OperationModel
from botocore.model import OperationNotFoundError as OperationNotFoundError
from botocore.model import ServiceModel
from botocore.utils import CachedProperty as CachedProperty

class EndpointDiscoveryException(BotoCoreError): ...

class EndpointDiscoveryRequired(EndpointDiscoveryException):
    fmt: str = ...

class EndpointDiscoveryRefreshFailed(EndpointDiscoveryException):
    fmt: str = ...

def block_endpoint_discovery_required_operations(model: Any, **kwargs: Any) -> None: ...

class EndpointDiscoveryModel:
    def __init__(self, service_model: ServiceModel) -> None: ...
    @CachedProperty
    def discovery_operation_name(self) -> str: ...
    @CachedProperty
    def discovery_operation_keys(self) -> List[str]: ...
    def discovery_required_for(self, operation_name: str) -> bool: ...
    def discovery_operation_kwargs(self, **kwargs: Any) -> Dict[str, Any]: ...
    def gather_identifiers(self, operation: OperationModel, params: Any) -> Dict[str, Any]: ...

class EndpointDiscoveryManager:
    def __init__(
        self,
        client: Any,
        cache: Optional[Any] = ...,
        current_time: Optional[Any] = ...,
        always_discover: bool = ...,
    ) -> None: ...
    def gather_identifiers(self, operation: Any, params: Any) -> Any: ...
    def delete_endpoints(self, **kwargs: Any) -> None: ...
    def describe_endpoint(self, **kwargs: Any) -> Any: ...

class EndpointDiscoveryHandler:
    def __init__(self, manager: Any) -> None: ...
    def register(self, events: Any, service_id: Any) -> None: ...
    def gather_identifiers(self, params: Any, model: Any, context: Any, **kwargs: Any) -> None: ...
    def discover_endpoint(self, request: Any, operation_name: Any, **kwargs: Any) -> None: ...
    def handle_retries(
        self, request_dict: Any, response: Any, operation: Any, **kwargs: Any
    ) -> Any: ...



from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Tuple, Type

from botocore import BOTOCORE_ROOT as BOTOCORE_ROOT
from botocore.compat import OrderedDict as OrderedDict
from botocore.exceptions import DataNotFoundError as DataNotFoundError
from botocore.exceptions import UnknownServiceError as UnknownServiceError
from botocore.utils import deep_merge as deep_merge

HAS_GZIP: bool

def instance_cache(func: Callable[..., Any]) -> Callable[..., Any]: ...

class JSONFileLoader:
    def exists(self, file_path: str) -> bool: ...
    def load_file(self, file_path: str) -> Any: ...

def create_loader(search_path_string: Optional[str] = ...) -> "Loader": ...

class Loader:
    FILE_LOADER_CLASS: Type[JSONFileLoader] = ...
    BUILTIN_DATA_PATH: str = ...
    CUSTOMER_DATA_PATH: str = ...
    BUILTIN_EXTRAS_TYPES: List[str] = ...
    def __init__(
        self,
        extra_search_paths: Optional[Iterable[str]] = ...,
        file_loader: Optional[JSONFileLoader] = ...,
        cache: Optional[Any] = ...,
        include_default_search_paths: bool = ...,
        include_default_extras: bool = ...,
    ) -> None:
        self.file_loader: JSONFileLoader
    @property
    def search_paths(self) -> List[str]: ...
    @property
    def extras_types(self) -> List[str]: ...
    def list_available_services(self, type_name: str) -> List[str]: ...
    def determine_latest_version(self, service_name: str, type_name: str) -> str: ...
    def list_api_versions(self, service_name: str, type_name: str) -> List[str]: ...
    def load_data_with_path(self, name: str) -> Tuple[Dict[str, Any], str]: ...
    def load_service_model(
        self, service_name: str, type_name: str, api_version: Optional[Any] = ...
    ) -> Any: ...
    def load_data(self, name: str) -> Any: ...
    def is_builtin_path(self, path: str) -> bool: ...

class ExtrasProcessor:
    def process(
        self, original_model: Mapping[str, Any], extra_models: Iterable[Mapping[str, Any]]
    ) -> None: ...
